#include "DXUT.h"
#include "MyFirstDX10App.h"

MyFirstDX10App::MyFirstDX10App(void)
{
}

MyFirstDX10App::~MyFirstDX10App(void)
{
}

HRESULT MyFirstDX10App::OnCreateDevice( const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc )
{
	DX10Framework::OnCreateDevice(pBackBufferSurfaceDesc);

	//-----------------------------------------------------------------------------
    // Create the effect
    ID3D10Blob* compilationErrors;	
	if(FAILED(
		D3DX10CreateEffectFromFile(
		L"constant.fx", NULL, NULL, "fx_4_0", D3D10_SHADER_DEBUG | D3D10_SHADER_SKIP_OPTIMIZATION, 0, _device,
		NULL, NULL, &_effect, &compilationErrors, NULL))) {
			MessageBoxA( NULL, (LPSTR)compilationErrors->GetBufferPointer(),
				"Failed to load effect file!", MB_OK);
			exit(-1);
	}

    // Obtain the technique
    _technique = _effect->GetTechniqueByName( "Render" );


	//-----------------------------------------------------------------------------
	// Define the input layout
	D3D10_INPUT_ELEMENT_DESC layout[] = {
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },  
	};
	UINT numElements = sizeof(layout)/sizeof(layout[0]);

	
	//-----------------------------------------------------------------------------
	// Create the input layout
	D3D10_PASS_DESC passDesc;
	_technique->GetPassByIndex( 0 )->GetDesc( &passDesc );
	if( FAILED( _device->CreateInputLayout( layout, numElements, passDesc.pIAInputSignature, 
			passDesc.IAInputSignatureSize, &_inputLayout ) ) )
			return S_FALSE;
	// Set the input layout
	_device->IASetInputLayout( _inputLayout );


	//-----------------------------------------------------------------------------
	// Create vertex buffer
	SimpleVertex vertices[] =
	{
		D3DXVECTOR3( 0.0f, 0.5f, 0.5f ),
		D3DXVECTOR3( 0.5f, -0.5f, 0.5f ),
		D3DXVECTOR3( -0.5f, -0.5f, 0.5f ),
	};
	D3D10_BUFFER_DESC bd;
	bd.Usage = D3D10_USAGE_DEFAULT;
	bd.ByteWidth = sizeof( SimpleVertex ) * 3;
	bd.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bd.CPUAccessFlags = 0;
	bd.MiscFlags = 0;
	D3D10_SUBRESOURCE_DATA initData;
	initData.pSysMem = vertices;
	if( FAILED( _device->CreateBuffer( &bd, &initData, &_vertexBuffer ) ) )
		return FALSE;

	// Set vertex buffer
	UINT stride = sizeof( SimpleVertex );
	UINT offset = 0;
	_device->IASetVertexBuffers( 0, 1, &_vertexBuffer, &stride, &offset );

	//-----------------------------------------------------------------------------
	_device->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

	return S_OK;
}

void MyFirstDX10App::OnDestroyDevice()
{
	_effect->Release();
	_vertexBuffer->Release();
	_inputLayout->Release();

	DX10Framework::OnDestroyDevice();
}

void MyFirstDX10App::OnFrameRender( double fTime, float fElapsedTime )
{
	float clearColor[4] = { 0.176f, 0.196f, 0.667f, 0.0f };
	_device->ClearRenderTargetView( _defaultRenderTargetView, clearColor );
	_device->ClearDepthStencilView( _defaultDepthStencilView, D3D10_CLEAR_DEPTH, 1.0f, 0);

	// Render a triangle
	D3D10_TECHNIQUE_DESC techDesc;
	_technique->GetDesc( &techDesc );
	for( UINT p = 0; p < techDesc.Passes; ++p )
	{
		_technique->GetPassByIndex( p )->Apply(0);
		_device->Draw( 3, 0 );
	}

}
